/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvControlVar.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_NOUPOUcB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0X1CHOTh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pDkN9u5N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NoNAHKbL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7JgUzTQm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_t05OooZg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3JQWwyrV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CVEb5ALR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vqs7c0Cf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_i6d6nM10_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6v4ki8Lt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1gc6Ub6k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_JR9wgOse_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_yVqowirh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4FnTpi3I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_oS16fdqa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_AqGhzpbw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_B3Dnn5Ik_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_v0P6YGOr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_mCkYHfoX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_fGFLsuBw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Mgoq3sVi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_uNy2sBPN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_tiMpCt4V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Fy4OiiRO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_e2Ja33qM_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_16A9VCEx;
  SignalLine sLine_W8WcJ921;
  SignalLine sLine_gCcNoxzT;
  ControlBinop cBinop_NOUPOUcB;
  ControlBinop cBinop_0X1CHOTh;
  ControlBinop cBinop_pDkN9u5N;
  ControlBinop cBinop_NoNAHKbL;
  ControlBinop cBinop_3JQWwyrV;
  ControlBinop cBinop_vqs7c0Cf;
  ControlBinop cBinop_i6d6nM10;
  ControlBinop cBinop_6v4ki8Lt;
  ControlVar cVar_1gc6Ub6k;
  ControlVar cVar_JR9wgOse;
  SignalVarf sVarf_KuCJb4dN;
  SignalVarf sVarf_AuhbpzPq;
  SignalVarf sVarf_pVAtazrq;
  SignalVarf sVarf_qwv20bV7;
  ControlBinop cBinop_B3Dnn5Ik;
  ControlBinop cBinop_Mgoq3sVi;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
